"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const change_case_1 = require("change-case");
const graphql_1 = require("graphql");
class FieldObject {
    constructor(field, keyPath, document) {
        this.field = field;
        this.keyPath = keyPath;
        this.document = document;
    }
    get name() {
        const { document, keyPath, compositeType } = this;
        return document.hasMultipleOfType(compositeType)
            ? `${change_case_1.pascal(keyPath.join(' '))}${compositeType.name}`
            : compositeType.name;
    }
    get compositeType() {
        return getNestedCompositeType(this.field.type);
    }
}
exports.FieldObject = FieldObject;
class Document {
    constructor(name, { fields, inlineFragments = [] }) {
        this.name = name;
        this.fieldObjects = [];
        this.typesUsed = new Map();
        this.fieldMap = new Map();
        for (const field of fields) {
            this.collectField(field);
        }
        for (const inlineFragment of inlineFragments) {
            this.collectFragment(inlineFragment);
        }
    }
    hasMultipleOfType(type) {
        return (this.typesUsed.get(type) || 0) > 1;
    }
    fieldObjectForField(field) {
        return this.fieldMap.get(field);
    }
    collectFragment(fragment, keyPath = []) {
        const { inlineFragments = [], fields } = fragment;
        for (const field of fields) {
            this.collectField(field, keyPath);
        }
        for (const inlineFragment of inlineFragments) {
            this.collectFragment(inlineFragment, keyPath);
        }
    }
    collectField(field, parentKeyPath = []) {
        if (field.fields == null) {
            return;
        }
        const newKeyPath = [...parentKeyPath, field.responseName];
        const fieldObject = new FieldObject(field, newKeyPath, this);
        const usedSoFar = this.typesUsed.get(fieldObject.compositeType) || 0;
        this.typesUsed.set(fieldObject.compositeType, usedSoFar + 1);
        this.fieldObjects.unshift(fieldObject);
        this.fieldMap.set(field, fieldObject);
        for (const subfield of field.fields) {
            this.collectField(subfield, newKeyPath);
        }
        for (const inlineFragment of (field.inlineFragments || [])) {
            this.collectFragment(inlineFragment, newKeyPath);
        }
    }
}
exports.default = Document;
function getNestedCompositeType(type) {
    let finalType = type;
    while (finalType instanceof graphql_1.GraphQLNonNull || finalType instanceof graphql_1.GraphQLList) {
        finalType = finalType.ofType;
    }
    if (!graphql_1.isCompositeType(finalType)) {
        throw new Error(`Expected a composite type, but received ${finalType.toString()}`);
    }
    return finalType;
}
